#import testInteractive

import gglobals
import time, gtk
gglobals.gourmetdir = '/tmp/'
gglobals.dbargs['file'] = '/tmp/recipes.db'

import GourmetRecipeManager

def add_save_and_check (rc, lines_groups_and_dc):
    added = []
    for l,g,dc in lines_groups_and_dc:
        added.append(rc.add_ingredient_from_line(l,group_iter=g))
    print 'added',added
    print 'turn into refs'
    added = [rc.ingtree_ui.ingController.get_persistent_ref_from_iter(i) for i in added]
    print 'added',added
    print 'This turns back into...'
    print [rc.ingtree_ui.ingController.get_iter_from_persistent_ref(r) for r in added]
    rc.saveEditsCB()
    print 'after saving...'
    print [rc.ingtree_ui.ingController.get_iter_from_persistent_ref(r) for r in added]
    ings = rc.rd.get_ings(rc.current_rec)
    check_ings([i[2] for i in lines_groups_and_dc],ings)
    return added

def check_ings (check_dics,ings):
    """Given a list of dictionaries of properties to check and
    ingredients, check that our ingredients have those properties.  We
    assume our check_dics refer to the last ingredients in the list
    ings
    """
    n = -1
    check_dics.reverse()
    for dic in check_dics:
        ings[n]
        for k,v in dic.items():
            try:
                assert(getattr(ings[n],k)==v)
            except AssertionError:
                print 'Failed assertion',n,k,v,ings[n]
                print 'We are looking for: '
                for d in check_dics: print ' ',d
                print 'in:'
                for a,u,i in [(i.amount,i.unit,i.item) for i in ings]: print ' ',a,u,i
                print 'we are at ',n,ings[n].amount,ings[n],ings[n].unit,ings[n].item
                print 'and we are failing to find ',k,v
                raise
        n -= 1

def test_ing_editing (rc):
    """Handed a recipe card, test ingredient editing"""
    # Add some ingredients in a group...
    rc.show_edit(tab=rc.NOTEBOOK_ING_PAGE)        
    g = rc.ingtree_ui.ingController.add_group('Foo bar')
    add_save_and_check(
        rc,
        [['1 c. sugar',g,
         {'amount':1,'unit':'c.','item':'sugar','inggroup':'Foo bar'}
         ],
        ['1 c. silly; chopped and sorted',g,
         {'amount':1,'unit':'c.','ingkey':'silly','inggroup':'Foo bar'},
         ],
        ['1 lb. very silly',g,
         {'amount':1,'unit':'lb.','item':'very silly','inggroup':'Foo bar'},
         ],
        ['1 tbs. extraordinarily silly',g,
         {'amount':1,'unit':'tbs.','item':'extraordinarily silly','inggroup':'Foo bar'}
         ],]
        )
    return g
    
def test_ing_undo (rc):
    #g = test_ing_editing(rc)
    #rc.ingtree_ui.ingController.delete_iters(g)
    #rc.undo.emit('activate')
    ings_groups_and_dcs = [['1 c. oil',None,{'amount':1,'unit':'c.','item':'oil'}],
         ['1 oz. peanuts',None,{'amount':1,'unit':'oz.','item':'peanuts'}],
         ]
    refs = add_save_and_check(
        rc,
        ings_groups_and_dcs
        )
    print 'refs',refs
    rc.ingtree_ui.ingController.do_delete_iters(refs)
    # Saving our edits...
    rc.saveEditsCB()
    try:
        ii = rc.rd.get_ings(rc.current_rec)
        check_ings(
            [i[2] for i in ings_groups_and_dcs],
            ii
            )
    except AssertionError:
        print 'Deletion worked!' # we expect an assertion error
    else:
        print [i[2] for i in ings_groups_and_dcs]
        print 'corresponds to'
        print [(i.amount,i.unit,i.item) for i in ii]
        raise "Ings Not Deleted!"
    # Undo after save...
    rc.undo.emit('activate')
    rc.saveEditsCB()

    # Check that our ingredients have been put back properly by the undo action!
    check_ings(
        [i[2] for i in ings_groups_and_dcs],
        rc.rd.get_ings(rc.current_rec)
        )
    print 'Undeletion worked!'

def test_undo_save_sensitivity (rc, verbose=True):
    rc.saveEditsCB()
    try:
        assert(not rc.save.get_sensitive())
    except:
        print 'FAILURE: SAVE Button not properly desensitized after save'; raise
    else:
        if verbose: print 'SAVE Button properly desensitized'
    for widget,value in [('rating',8),
                         ('preptime',30*60),
                         ('cooktime',60*60),
                         ('title','Foo bar'),
                         ('cuisine','Mexican'),
                         ('category','Entree')                         
                         ]:
        if verbose: print 'TESTING ',widget
        if type(value)==int:
            orig_value = rc.rw[widget].get_value()
            rc.rw[widget].set_value(value)
            get_method = rc.rw[widget].get_value
            if verbose: print 'Set with set_value(',value,')'
        elif widget in rc.reccom:
            orig_value = rc.rw[widget].entry.get_text()
            rc.rw[widget].entry.set_text(value)
            get_method = rc.rw[widget].entry.get_text
            if verbose: print 'Set with entry.set_text(',value,')'            
        else:
            orig_value = rc.rw[widget].get_text()
            rc.rw[widget].set_text(value)
            get_method = rc.rw[widget].get_text
            if verbose: print 'Set with set_text(',value,')'                        
        try: assert(get_method()==value)
        except: print '''Value not set properly for %s
        is %s, should be %s'''%(
            widget,get_method(),value
            ); raise
        try:
            assert(rc.save.get_sensitive())
        except:
            print 'Save not sensitized after setting %s'%widget; raise
        else:
            if verbose: print 'Save sensitized properly after setting %s'%widget
        rc.undo.emit('activate')
        if orig_value and type(value)!=int: rc.undo.emit('activate') # Blank text, then fill it
        try:
            assert(get_method()==orig_value)
        except:
            print '''Value not unset properly on for %s
            Should have been set to %s'''%(widget,orig_value); raise
        else:
            if verbose: print 'Value set properly for %s after Undo'%widget
        try:
            assert(not rc.save.get_sensitive())
        except:
            print 'Save not desensitized after unsetting %s'%widget; raise
        else:
            if verbose: print 'Save desensitized correctly after unsetting %s'%widget
        rc.redo.emit('activate')
        if orig_value and type(value)!=int: rc.redo.emit('activate') # Blank text, then fill it        
        try: assert(get_method()==value)
        except: print '''Value not set properly for REDO %s
        is %s, should be %s'''%(
            widget,get_method(),value
            ); raise
        try:
            assert(rc.save.get_sensitive())
        except:
            print 'Save not sensitized after setting %s via REDO'%widget; raise
        rc.undo.emit('activate')
        if orig_value and type(value)!=int: rc.undo.emit('activate') # Blank text, then fill it
        try:
            assert(get_method()==orig_value)
        except:
            print '''Value not unset properly on for %s UNDO->REDO->UNDO
            Should have been set to %s'''%(widget,orig_value); raise
        else:
            if verbose: print 'Value set properly for %s after Undo->Redo->Undo'%widget
        try:
            assert(not rc.save.get_sensitive())
        except:
            print 'Save not desensitized after undo->redo->undo %s'%widget; raise
        else:
            if verbose: print 'Save desensitized correctly after undo->redo->undo %s'%widget

rg = GourmetRecipeManager.RecGui()
rg.newRecCard()
while gtk.events_pending(): gtk.main_iteration()
rec_id,rec_card = rg.rc.items()[0]

try:
    #'break'+1
    test_ing_editing(rec_card)
    test_ing_undo(rec_card)
    test_undo_save_sensitivity(rec_card)
except:
    import traceback; traceback.print_exc()
    gtk.main()
else:
    rec_card.hide()
    
    
